#if defined _revive_teammates_const_included
  #endinput
#endif
#define _revive_teammates_const_included

#if !defined _json_included
	#include <json>
#endif

// Corpse classname
stock const CORPSE_CLASSNAME[] = "info_corpse";

// Corpse default collision
stock const Float:DEFAULT_CORPSE_MINS[3] = { -24.0, -24.0, -24.0 };
stock const Float:DEFAULT_CORPSE_MAXS[3] = { 24.0, 24.0, 24.0 };

// Corpse max distance
stock const Float:CORPSE_MAX_RANGE = 128.0;

stock const RT_VERSION_STR[] = "3.1.86";
// Plugin version
stock const RT_VERSION_NUM = 3186;
stock const RT_SUFFIX[] = "RT";

stock const EntVars:var_activator = var_euser1;
stock const EntVars:var_timer = var_fuser1;

// Helper macros
#define VALID_PLAYER(%0)		(1 <= %0 <= MaxClients)
#define VALID_TEAM(%0)			(TEAM_TERRORIST <= get_member(%0, m_iTeam) <= TEAM_CT)
#define VALID_RELATION(%0)		(GR_NOTTEAMMATE <= %0 <= GR_ENEMY)

#define MOD(%0,%1)				((((%0) % (%1)) + (%1)) % (%1))
#define CLAMP(%0,%1,%2)			(max(%1, min(%2, %0)))

// Return type
enum
{
	RT_CONTINUE = 0,
	RT_SUPERCEDE
};

/*
	For RT_StartSneaking()
*/
enum
{
	CORPSE_CUSTOM = 0,
	CORPSE_DEFAULT,
	CORPSE_BLOCK
};

/*
	For RT_RelationShipStatus()
*/
enum RelationStatus
{
	GR_NOTTEAMMATE = 0,
	GR_TEAMMATE,
	GR_ENEMY
};

stock json_cvar_integer(const JSON:object, const value[], const def, const description[], bool:has_min = false, min_val = 0, bool:has_max = false, max_val = 0, &cvar)
{
	new JSON:jsonCvar;
	
	if (json_object_has_value(object, value, JSONObject))
	{
		jsonCvar = json_object_get_value(object, value);
		cvar = json_object_get_number(jsonCvar, "#");
		
		if (has_max && cvar > max_val)
			cvar = max_val;
		else if (has_min && cvar < min_val)
			cvar = min_val;
	}
	else
	{
		jsonCvar = json_init_object();
		json_object_set_number(jsonCvar, "#", cvar = def);
	}
	
	json_cvar_set_description(jsonCvar, description);
	// json_object_set_number(jsonCvar, "default", def);
	
	if (has_min) {
		json_object_set_number(jsonCvar, "min", min_val);
	}
	if (has_max) {
		json_object_set_number(jsonCvar, "max", max_val);
	}
	
	json_object_set_value(object, value, jsonCvar);
	json_free(jsonCvar);
}

stock json_cvar_float(const JSON:object, const value[], const Float:def, const description[], bool:has_min = false, Float:min_val = 0.0, bool:has_max = false, Float:max_val = 0.0, &Float:cvar)
{
	new JSON:jsonCvar;
	
	if (json_object_has_value(object, value, JSONObject))
	{
		jsonCvar = json_object_get_value(object, value);
		cvar = json_object_get_real(jsonCvar, "#");
		
		if (has_max && cvar > max_val)
			cvar = max_val;
		else if (has_min && cvar < min_val)
			cvar = min_val;
	}
	else
	{
		jsonCvar = json_init_object();
		json_object_set_real(jsonCvar, "#", cvar = def);
	}
	
	json_cvar_set_description(jsonCvar, description);
	// json_object_set_real(jsonCvar, "default", def);
	
	if (has_min) {
		json_object_set_real(jsonCvar, "min", min_val);
	}
	if (has_max) {
		json_object_set_real(jsonCvar, "max", max_val);
	}
	
	json_object_set_value(object, value, jsonCvar);
	json_free(jsonCvar);
}

stock json_cvar_string(const JSON:object, const value[], const def[], const description[], cvar[], len, bool:precache = false)
{
	new JSON:jsonCvar;
	
	if (json_object_has_value(object, value, JSONObject))
	{
		jsonCvar = json_object_get_value(object, value);
		json_object_get_string(jsonCvar, "#", cvar, len);
	}
	else
	{
		jsonCvar = json_init_object();
		json_object_set_string(jsonCvar, "#", def);
		copy(cvar, len, def);
	}
	
	json_cvar_set_description(jsonCvar, description);
	// json_object_set_string(jsonCvar, "default", def);
	
	// Need tested
	if (precache && cvar[0])
	{
		new file_length = strlen(cvar);
		switch (cvar[--file_length])
		{
		case '3':
			precache_generic(cvar);
		case 'v':
			precache_sound(cvar);
		case 'l', 'r':
			precache_model(cvar);
		}
	}
	
	json_object_set_value(object, value, jsonCvar);
	json_free(jsonCvar);
}

stock json_cvar_set_description(JSON:cvar, const description[])
{
	new desc[512];
	new reloc_idx, idx;
	new JSON:jsonDescription = json_init_array();

	while ((idx = split_string(description[reloc_idx], "^n", desc, charsmax(desc))) != -1)
	{
		reloc_idx += idx;
		json_array_append_string(jsonDescription, desc);
	}
	
	// FIX: Last arg
	if (reloc_idx > 0)
	{
		json_array_append_string(jsonDescription, description[reloc_idx]);
		json_object_set_value(cvar, "description", jsonDescription);
	}
	else
	{
		json_object_set_string(cvar, "description", description);
	}
	
	json_free(jsonDescription);
}

stock read_flags_ex(const flags[])
{
	if (!flags[0] || flags[0] == '0')
	{
		return 0;
	}
	
	return read_flags(flags);
}

// Same as fm_entity_range (fakemeta_util.inc)
stock Float:rg_entity_range(ent1, ent2)
{
	static Float:origin1[3], Float:origin2[3];
	get_entvar(ent1, var_origin, origin1);
	get_entvar(ent2, var_origin, origin2);

	return get_distance_f(origin1, origin2);
}

// For JSON colored format
stock replace_color_tag(string[], len)
{
	replace_string(string, len, "!y", "^1");
	replace_string(string, len, "!t", "^3");
	replace_string(string, len, "!g", "^4");
}